import {GetServerSideProps, NextPage} from 'next'
import {useEffect, useState} from 'react';
import {models} from '@tonality/library-common'
import {useUser} from '@auth0/nextjs-auth0'
import CreateRingtoneTemplate from '../../../../components/templates/CreateRingtone'
import RingtoneClient from '../../../../modules/ringtone/client'
import {getSession, withPageAuthRequired} from '@auth0/nextjs-auth0';
import WaveOscillator from '../../../../utils/sound/WaveOscillator';
import SoundManager from '../../../../utils/sound/SoundManager';
import ComposerClient from '../../../../modules/composer/client';

type Props = {
	user: models.User,
	composerRingtones: models.Ringtone[],
}

const MyCreateRingtonePage: NextPage<Props> = ({
	user,
	composerRingtones,
}) => {
	const [hydrated, setHydrated] = useState(false)
	const [ringtoneClient, setRingtoneClient] = useState<RingtoneClient>(null)
	const [composerClient, setComposerClient] = useState<ComposerClient>(null)
	const theUser = useUser()

	useEffect(() => {
		setHydrated(true)
	}, [])

	useEffect(() => {
		setRingtoneClient(new RingtoneClient(process.env.NEXT_PUBLIC_API_BASE_URL))
	}, [hydrated])

	useEffect(() => {
		const audioContext = new AudioContext()
		const gainNode = audioContext.createGain()
		gainNode.gain.value = 0.05
		gainNode.connect(audioContext.destination)
		const oscillator = new WaveOscillator(audioContext, gainNode)
		const soundManager = new SoundManager(oscillator)
		setComposerClient(new ComposerClient(soundManager))
	}, [hydrated])

	return (
		<CreateRingtoneTemplate
			composer={user}
			composerRingtones={composerRingtones}
			addNote={composerClient ? composerClient.addNote : undefined}
			addRest={composerClient ? composerClient.addRest : undefined}
			togglePlayback={composerClient ? composerClient.togglePlayback : undefined}
			updateSong={composerClient ? composerClient.updateSong : undefined}
			updateTempo={composerClient ? composerClient.updateTempo : undefined}
			updateView={composerClient ? composerClient.updateView : undefined}
			onSubmit={ringtoneClient ? ringtoneClient.save : undefined}
			play={composerClient ? composerClient.play : undefined}
		/>
	)
}

export const getServerSideProps: GetServerSideProps = withPageAuthRequired({
	getServerSideProps: async (ctx) => {
		const session = getSession(ctx.req, ctx.res)
		const { user } = session
		const composerRingtones = []

		return {
			props: {
				user: {
					id: user.sub,
					name: user.nickname,
					bio: '',
				},
				composerRingtones,
			},
		}
	},
	returnTo: '/my/create/ringtone'
})

export default MyCreateRingtonePage


/*
8g4 8g4 4a4 4g4 4c5 2b4 8g4 8g4 4a4 4g4 4d5 2c5 8g4 8g4 4g5 4e5 8c5 8c5 4b4 4a4 8f5 8f5 4e5 4c5 4d5 2c5.
 */

/*
32c4 32g4 32e5 32e4 32c5 32g5 32g4 32e5 32c6 32c5 32g5 32e6 32e5 32c6 32g6 32g5 32e6 32c7 32c6 32g6 32e7 32e6 32c7 32g7
 */
