import {inject, singleton} from 'tsyringe';
import UserService from '../user/service';
import PasswordService from '../password/service';
import CredentialsService from '../credentials/service';
import Credentials from '../credentials/type';

export default interface AuthService {
	logIn(username: string, password: string): Promise<Credentials>
}

@singleton()
export class AuthServiceImpl implements AuthService {
	constructor(
		@inject('UserService')
		private readonly userService: UserService,
		@inject('PasswordService')
		private readonly passwordService: PasswordService,
		@inject('CredentialsService')
		private readonly credentialsService: CredentialsService,
	) {}

	async logIn(username: string, password: string): Promise<Credentials> {
		const user = await this.userService.getByUsername(username)
		if (!user) {
			throw new Error('Invalid credentials.')
		}

		const valid = this.passwordService.compare(password, user.password)
		if (!valid) {
			throw new Error('Invalid credentials.')
		}

		const credentials = await this.credentialsService.request()
		credentials.profile = await this.userService.getProfileByUsername(username)
		return credentials
	}
}
