import {URL} from 'url';
import unfetch from 'unfetch';
import Credentials from './type';
import {singleton} from 'tsyringe';

export default interface CredentialsService {
	request(): Promise<Credentials>
}

@singleton()
export class CredentialsServiceImpl {
	async request(): Promise<Credentials> {
		const tokenUrl = new URL('/oauth/token', process.env.AUTH0_DOMAIN)
		const audienceUrl = new URL('/api/v2', process.env.AUTH0_DOMAIN)
		const response = await unfetch(tokenUrl.toString(), {
			method: 'POST',
			body: JSON.stringify({
				client_id: process.env.AUTH0_CLIENT_ID,
				client_secret: process.env.AUTH0_SECRET,
				audience: audienceUrl.toString(),
				grant_type: 'client_credentials',
			})
		})
		if (!response.ok) {
			throw new Error('Unable to request credentials.')
		}
		const { access_token: accessToken, expires_in: expiresIn, token_type: tokenType } = await response.json()
		return {
			accessToken,
			expiresIn,
			tokenType,
		}
	}
}
