import { genSalt, hash, compare } from 'bcrypt';
import {inject, singleton} from 'tsyringe';

export default interface PasswordService {
	hash(password: string): Promise<string>
	compare(password: string, hash: string): Promise<boolean>
}

@singleton()
export class PasswordServiceImpl implements PasswordService {
	constructor(
		@inject('PASSWORD_GEN_SALT_ROUNDS')
		private readonly passwordGenSaltRounds: number,
	) {}

	async hash(password: string) {
		const salt = await genSalt(this.passwordGenSaltRounds)
		return await hash(password, salt)
	}

	async compare(password: string, hash: string) {
		return await compare(password, hash)
	}
}
